<?php
/**
 * webEdition CMS
 *
 * $Rev: 10641 $
 * $Author: andreaswitt $
 * $Date: 2015-10-21 11:11:50 +0200 (Wed, 21 Oct 2015) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */
require_once(WE_INCLUDES_PATH . 'we_tag.inc.php');

/* the parent class for documents */

class we_document extends we_root{
//Extension of the document
	var $Extension = '';
//Array of possible filename extensions for the document
	var $Extensions;
	var $Published = 0;
	var $Language = '';
//If the file should only be saved in the db
	var $IsDynamic = 0;
	var $schedArr = array();
//Categories of the document
	var $Category = '';
	protected $oldCategory = '';
	var $IsSearchable = 0;
	var $InGlossar = 0;
//var $NavigationItems = '';
	private $DocStream = '';
	public $parseFile = 1;
// persistent in we_object*, since saved in class
// temporary in document, holds Paths to stylesheets from we:css-tags that are user by tinyMCE
	var $CSS = array();
	/* this array is used, to store document specific data for a page */
	protected $editorSaves = array();

	function __construct(){
		parent::__construct();
		array_push($this->persistent_slots, 'Extension', 'IsDynamic', 'Published', 'Category', 'IsSearchable', 'InGlossar', 'Language', 'schedArr', 'parseFile', 'editorSaves');
		$this->Table = FILE_TABLE;
		if(defined('WE_SIDEBAR')){
			$this->InWebEdition = true;
		}
	}

	function copyDoc($id){
		if($id){
			$tmp = $this->ClassName;
			$doc = new $tmp();
			$doc->InitByID($id, $this->Table);
			$parentIDMerk = $doc->ParentID;
			if($this->ID == 0){
				foreach($this->persistent_slots as $name){
					if($name != 'elements' && in_array($name, array_keys(get_object_vars($doc)))){
						$this->{$name} = $doc->{$name};
					}
				}
				$this->Published = 0;
				if(isset($doc->Category)){
					$this->Category = $doc->Category;
				}
				$this->CreationDate = time();
				$this->CreatorID = (isset($_SESSION['user']) ? $_SESSION['user']['ID'] : 0);

				$this->ID = 0;
				$this->OldPath = '';
				$this->Filename .= '_copy';
				$this->Text = $this->Filename . $this->Extension;
				$this->setParentID($parentIDMerk);
				$this->Path = $this->ParentPath . $this->Text;
				$this->OldPath = $this->Path;
			}
			$this->elements = $doc->elements;
			foreach($this->elements as $n => $e){
				$this->elements[$n]['cid'] = 0;
			}
			$this->EditPageNr = we_base_constants::WE_EDITPAGE_PROPERTIES;
			$this->InWebEdition = true;
			if(isset($this->documentCustomerFilter)){
				$this->documentCustomerFilter = $doc->documentCustomerFilter;
			}
		}
	}

	/** gets the filesize of the document */
	function getFilesize(){
		return strlen($this->getElement('data'));
	}

// returns the whole document Alias - don't remove
	function getDocument($we_editmode = 0, $baseHref = 0, $we_transaction = ''){
		return $this->i_getDocument();
	}

	private function initLanguageFromParent(){
		$ParentID = $this->ParentID;
		$i = 0;
		while(!$this->Language){
			if($ParentID == 0 || $i > 20){
				we_loadLanguageConfig();
				$this->Language = parent::getDefaultLanguage();
				if(empty($this->Language)){
					$this->Language = 'de_DE';
				}
			} else {
				$this->DB_WE->query('SELECT Language, ParentID FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($ParentID));

				while($this->DB_WE->next_record()){
					$ParentID = $this->DB_WE->f('ParentID');
					$this->Language = $this->DB_WE->f('Language');
				}
			}
			$i++;
		}
	}

	/*
	 * Form Functions
	 */

	function formInGlossar(){
		return (we_base_moduleInfo::we_getModuleNameByContentType('glossary') === 'glossary' ?
				we_html_forms::checkboxWithHidden((bool) $this->InGlossar, 'we_' . $this->Name . '_InGlossar', g_l('weClass', '[InGlossar]'), false, 'defaultfont', '_EditorFrame.setEditorIsHot(true);') :
				'');
	}

	function formIsSearchable(){
		return we_html_forms::checkboxWithHidden((bool) $this->IsSearchable, 'we_' . $this->Name . '_IsSearchable', g_l('weClass', '[IsSearchable]'), false, 'defaultfont', '_EditorFrame.setEditorIsHot(true);');
	}

	protected function formExtension2(){
		$doctype = isset($this->DocType) ? $this->DocType : '';

		if(!$this->ID && we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0) === 'load_editor' && $doctype == ''){ //	Neues Dokument oder Dokument ohne DocType
			switch($this->ContentType){
				case we_base_ContentTypes::HTML: //	is HTML-File
					$selected = DEFAULT_HTML_EXT;
					break;
				case we_base_ContentTypes::WEDOCUMENT: //	webEdition Document
					$selected = ($this->IsDynamic == 1 ? DEFAULT_DYNAMIC_EXT : DEFAULT_STATIC_EXT);
					break;
				default: //	no webEdition Document
					$selected = $this->Extension;
					break;
			}
		} else { //	bestehendes Dokument oder Dokument mit DocType
			$selected = $this->Extension;
		}
		return $this->Extensions ?
			we_html_tools::htmlFormElementTable(we_html_tools::getExtensionPopup('we_' . $this->Name . '_Extension', $selected, $this->Extensions, 100, 'onselect="_EditorFrame.setEditorIsHot(true);"', permissionhandler::hasPerm('EDIT_DOCEXTENSION')), g_l('weClass', '[extension]')) :
			we_html_element::htmlHidden(array(
				'name' => 'we_' . $this->Name . '_Extension',
				'value' => $selected
		));
	}

	public function formPath(){
		$disable = ( ($this->ContentType == we_base_ContentTypes::HTML || $this->ContentType == we_base_ContentTypes::WEDOCUMENT) && $this->Published);
		if($this->ContentType === we_base_ContentTypes::HTACESS){
			$vals = we_base_ContentTypes::inst()->getExtension($this->ContentType, true);
			$this->Filename = $this->Filename ? : current($vals);
			$filenameinput = $this->formSelectFromArray('', 'Filename', array_combine($vals, $vals), g_l('weClass', '[filename]'));
		} else {
			$filenameinput = $this->formInputField('', 'Filename', g_l('weClass', '[filename]'), 30, 388, 255, 'onchange="_EditorFrame.setEditorIsHot(true);if(self.pathOfDocumentChanged){pathOfDocumentChanged();}"');
		}
		return $disable ? ('<span class="defaultfont">' . $this->Path . '</span>') : '
<table border="0" cellpadding="0" cellspacing="0">
	<tr>
		<td>' . $filenameinput . '</td>
		<td></td>
		<td>' . $this->formExtension2() . '</td>
	</tr>
	<tr>
		<td>' . we_html_tools::getPixel(20, 4) . '</td>
		<td>' . we_html_tools::getPixel(20, 2) . '</td>
		<td>' . we_html_tools::getPixel(100, 2) . '</td>
	</tr>
	<tr><td colspan="3">' . $this->formDirChooser(388) . '</td></tr>
</table>';
	}

	function formMetaInfos(){
		return '
<table border="0" cellpadding="0" cellspacing="0">
	<tr><td colspan="2">' . $this->formInputField("txt", "Title", g_l('weClass', '[Title]'), 40, 508, "", "onchange=\"_EditorFrame.setEditorIsHot(true);\"") . '</td></tr>
	<tr><td>' . we_html_tools::getPixel(2, 4) . '</td></tr>
	<tr><td colspan="2">' . $this->formInputField("txt", "Description", g_l('weClass', '[Description]'), 40, 508, "", "onchange=\"_EditorFrame.setEditorIsHot(true);\"") . '</td></tr>
	<tr><td>' . we_html_tools::getPixel(2, 4) . '</td></tr>
	<tr><td colspan="2">' . $this->formInputField("txt", "Keywords", g_l('weClass', '[Keywords]'), 40, 508, "", "onchange=\"_EditorFrame.setEditorIsHot(true);\"") . '</td></tr>
</table>' .
			($this->ContentType == we_base_ContentTypes::IMAGE ? $this->formCharset(true) : '');
	}

	function formCategory(){
		$delallbut = we_html_button::create_button('delete_all', "javascript:we_cmd('delete_all_cats')", true, 0, 0, '', '', $this->Category ? false : true);
		$addbut = we_html_button::create_button('add', "javascript:we_cmd('openCatselector',-1,'" . CATEGORY_TABLE . "','','','opener.setScrollTo();fillIDs();opener.top.we_cmd(\\'add_cat\\',top.allIDs);')");
		$cats = new we_chooser_multiDir(508, $this->Category, 'delete_cat', we_html_button::create_button_table(array($delallbut, $addbut)), '', 'Icon,Path', CATEGORY_TABLE);
		$cats->extraDelFn = 'setScrollTo();';
		return $cats->get();
	}

	function formNavigation(){
		$isSee = $_SESSION['weS']['we_mode'] == we_base_constants::MODE_SEE;
		$navItems = $this->getNavigationItems();
		$addbut = we_html_button::create_button('add', "javascript:we_cmd('module_navigation_edit_navi',0)", true, 100, 22, '', '', (permissionhandler::hasPerm('EDIT_NAVIGATION') && $this->ID && $this->Published) ? false : true, false);

		if(permissionhandler::hasPerm('EDIT_NAVIGATION') && $isSee){
			$delallbut = we_html_button::create_button('delete_all', "javascript:if(confirm('" . g_l('navigation', '[dellall_question]') . "')) we_cmd('delete_all_navi')", true, 0, 0, "", "", (permissionhandler::hasPerm('EDIT_NAVIGATION') && $navItems) ? false : true);
		} else {
			$delallbut = '';
		}
		$navis = new we_chooser_multiFile(508, $navItems, 'delete_navi', we_html_button::create_button_table(array($delallbut, $addbut)), 'module_navigation_edit_navi', 'Icon,Path', NAVIGATION_TABLE);
		$navis->extraDelFn = 'setScrollTo();';
		$NoDelNavis = $navItems;
		foreach($NoDelNavis as $_path){
			$_id = path_to_id($_path, NAVIGATION_TABLE);
			$_naviItem = new we_navigation_navigation($_id);
			if(!$_naviItem->hasAnyChilds()){

				if(in_array($_path, $NoDelNavis)){
					$pos = array_search($_path, $NoDelNavis);
					unset($NoDelNavis[$pos]);
				}
			}
		}
		$navis->setDisabledDelItems($NoDelNavis, g_l('navigation', '[NoDeleteFromDocument]'));

		if(!$isSee || !permissionhandler::hasPerm('EDIT_NAVIGATION')){
			$navis->isEditable = false;
			$navis->CanDelete = false;
		}

		return we_html_button::create_state_changer() . $navis->get();
	}

	function addCat(array $ids){
		$cats = makeArrayFromCSV($this->Category);
		foreach($ids as $id){
			if($id && (!in_array($id, $cats))){
				$cats[] = $id;
			}
		}
		$this->Category = makeCSVFromArray($cats, true);
	}

	function delCat($id){
		$cats = makeArrayFromCSV($this->Category);
		if(in_array($id, $cats)){
			$pos = array_search($id, $cats);
			if($pos !== false || $pos == '0'){
				unset($cats[$pos]);
			}
		}
		$this->Category = makeCSVFromArray($cats, true);
	}

	function addNavi($id, $text, $parentid, $ordn){
		$text = urldecode($text); //Bug #3769
		if($this->ID){
			if(is_numeric($ordn)){
				$ordn--;
			}
			$_ord = ($ordn === 'end' ? -1 : (is_numeric($ordn) && $ordn > 0 ? $ordn : 0));

			$_ppath = id_to_path($parentid, NAVIGATION_TABLE);
			$_new_path = rtrim($_ppath, '/') . '/' . $text;
			$id = $id? : path_to_id($_new_path, NAVIGATION_TABLE);

			$_naviItem = new we_navigation_navigation($id);

			$_naviItem->Ordn = f('SELECT MAX(Ordn) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($parentid));
			$_naviItem->ParentID = $parentid;
			$_naviItem->LinkID = $this->ID;
			$_naviItem->Text = $text;
			$_naviItem->Path = $_new_path;
			if(NAVIGATION_ENTRIES_FROM_DOCUMENT){
				$_naviItem->Selection = we_navigation_navigation::SELECTION_STATIC;
				$_naviItem->SelectionType = we_navigation_navigation::STPYE_DOCLINK;
			} else {
				$_naviItem->Selection = we_navigation_navigation::SELECTION_NODYNAMIC;
				$_naviItem->SelectionType = we_navigation_navigation::STPYE_DOCTYPE;
				$_naviItem->IsFolder = 1;
				$charset = $_naviItem->findCharset($_naviItem->ParentID);
				$_naviItem->Charset = ($charset ? : (DEFAULT_CHARSET ? : $GLOBALS['WE_BACKENDCHARSET']));
			}

			$_naviItem->save();
			$_naviItem->reorderAbs($_ord);
		}
	}

	function delNavi($path){
		$path = urldecode($path); //Bug #3816
		$navis = $this->getNavigationItems();
		if(in_array($path, $navis)){
			$pos = array_search($path, $navis);
			if($pos !== false || $pos == 0){
				$_id = path_to_id($path, NAVIGATION_TABLE);
				$_naviItem = new we_navigation_navigation($_id);
				if(!$_naviItem->hasAnyChilds()){
					$_naviItem->delete();
					unset($navis[$pos]);
				}
			}
		}
	}

	function delAllNavi(){
		$navis = $this->getNavigationItems();
		foreach($navis as $_path){
			$_id = path_to_id($_path, NAVIGATION_TABLE);
			$_naviItem = new we_navigation_navigation($_id);
			if(!$_naviItem->hasAnyChilds()){
				$_naviItem->delete();
				if(in_array($_path, $navis)){
					$pos = array_search($_path, $navis);
					unset($navis[$pos]);
				}
			}
		}
	}

	/*
	 * internal functions
	 */

	function getParentIDFromParentPath(){
		$f = new we_folder();
		return ($f->initByPath($this->ParentPath) ? $f->ID : -1);
	}

	function addEntryToList($name, $number = 1){
		$list = $this->getElement($name);

		$listarray = $list ? unserialize($list) : array();
		$listarray = is_array($listarray) ? $listarray : array();

		for($f = 0; $f < $number; $f++){
			$content = $this->getElement($name, 'content');
			$new_nr = $this->getMaxListArrayNr($listarray) + 1;

// clear value
			$names = $this->getNamesFromContent($content);

			foreach($names as $_name){
				$this->setElement($_name . '_' . $new_nr, '');
			}

			$listarray[] = '_' . $new_nr;
		}
		$this->setElement($name, serialize(array_values($listarray)), 'block');
	}

	function getMaxListArrayNr(array $la){
		$maxnr = 0;
		foreach($la as $val){
			$nr = intval(str_replace('_', '', $val));
			$maxnr = max($maxnr, $nr);
		}
		return $maxnr;
	}

	function insertEntryAtList($name, $nr, $number = 1){
		$list = $this->getElement($name);

		$listarray = $list ? unserialize($list) : array();
		$listarray = is_array($listarray) ? $listarray : array();

		for($f = 0; $f < $number; $f++){

			$content = $this->getElement($name, 'content');
			$new_nr = $this->getMaxListArrayNr($listarray) + 1;
// clear value
			$names = $this->getNamesFromContent($content);
			foreach($names as $cur){
				$this->setElement($cur . '_' . $new_nr, '');
			}

			for($i = count($listarray); $i > $nr; $i--){
				$listarray[$i] = $listarray[$i - 1];
			}

			$listarray[$nr] = '_' . $new_nr;
		}

		$this->setElement($name, serialize(array_values($listarray)), 'block');
	}

	function upEntryAtList($name, $nr, $number = 1){
		$list = $this->getElement($name);
		if(!$list){
			t_e('failed');
			return;
		}
		$listarray = unserialize($list);
		$newPos = max($nr - $number, 0);
		$temp = $listarray[$newPos];
		$listarray[$newPos] = $listarray[$nr];
		$listarray[$nr] = $temp;

		$this->setElement($name, serialize($listarray), 'block');
	}

	function downEntryAtList($name, $nr, $number = 1){
		$list = $this->getElement($name);
		if(!$list){
			return;
		}
		$listarray = unserialize($list);
		$newPos = min($nr + $number, count($listarray) - 1);
		$temp = $listarray[$newPos];
		$listarray[$newPos] = $listarray[$nr];
		$listarray[$nr] = $temp;
		$this->setElement($name, serialize($listarray), 'block');
	}

	function removeEntryFromList($name, $nr, $names = '', $isBlock = false){
		$list = $this->getElement($name);
		$listarray = $list ? unserialize($list) : array();
		if(is_array($listarray)){
			if($isBlock){
				foreach(array_keys($this->elements) as $key){
					if(preg_match('/' . $names . '(__.*)*$/', $key)){// # Bug 6904
						unset($this->elements[$key]);
					}
				}
			} else {
				$namesArray = $names ? explode(',', $names) : array($names);
				foreach($namesArray as $element){
					unset($this->elements[$element . $listarray[$nr]]);
				}
			}
			if(is_array($listarray)){// Bug #4079
				unset($listarray[$nr]);
			}
		} else {
			$listarray = array();
		}

		$this->setElement($name, serialize(array_values($listarray)), 'block');
	}

	function addLinkToLinklist($name){
		$ll = new we_base_linklist($this->getElement($name));
		$ll->addLink();
		$this->setElement($name, $ll->getString(), 'linklist');
	}

	function upEntryAtLinklist($name, $nr){
		$ll = new we_base_linklist($this->getElement($name));
		$ll->upLink($nr);
		$this->setElement($name, $ll->getString(), 'linklist');
	}

	function downEntryAtLinklist($name, $nr){
		$ll = new we_base_linklist($this->getElement($name));
		$ll->downLink($nr);
		$this->setElement($name, $ll->getString(), 'linklist');
	}

	function insertLinkAtLinklist($name, $nr){
		$ll = new we_base_linklist($this->getElement($name));
		$ll->insertLink($nr);
		$this->setElement($name, $ll->getString(), 'linklist');
	}

	function removeLinkFromLinklist($name, $nr, $names = ''){
		$ll = new we_base_linklist($this->getElement($name));
		$ll->removeLink($nr, $names, $name);
		$this->setElement($name, $ll->getString(), 'linklist');
	}

	function changeLink($name){//FIXME: can we store info in bdid? add info on type, if it is object or file?
		if(!isset($_SESSION['weS']['WE_LINK'])){
			return;
		}
		$this->setElement($name, serialize($_SESSION['weS']['WE_LINK']), 'link');
		unset($_SESSION['weS']['WE_LINK']);
	}

	function changeLinklist($name){
		if(!isset($_SESSION['weS']['WE_LINKLIST'])){
			return;
		}
		$this->setElement($name, $_SESSION['weS']['WE_LINKLIST'], 'linklist');
		unset($_SESSION['weS']['WE_LINKLIST']);
	}

	function getNamesFromContent($content){
		$arr = $result = array();
		preg_match_all('/< ?we:[^>]+name="([^"]+)"[^>]*>/i', $content, $result, PREG_SET_ORDER);
		foreach($result as $val){
			$arr[] = $val[1];
		}
		return $arr;
	}

	function remove_image($name){
		unset($this->elements[$name]);
		unset($this->elements[$name . we_imageDocument::ALT_FIELD]);
		unset($this->elements[$name . we_imageDocument::TITLE_FIELD]);
	}

	/*
	 * public
	 */

	public function we_new(){
		parent::we_new();
		$this->i_setExtensions();
		if(is_array($this->Extensions) && $this->Extensions){
			$this->Extension = $this->Extensions[0];
		}
		if(!isset($GLOBALS['WE_IS_DYN']) && ($this->Table == FILE_TABLE || $this->Table == TEMPLATES_TABLE)){
			if(($ws = get_ws($this->Table))){
				$foo = makeArrayFromCSV($ws);
				if($foo){
					$this->setParentID(intval($foo[0]));
				}
			}
		}
	}

	private function i_setExtensions(){
		if($this->ContentType){
			$exts = we_base_ContentTypes::inst()->getExtension($this->ContentType);
			$this->Extensions = is_array($exts) && $exts ? $exts : array();
			$this->Extension = $this->Extension? : (!is_array($exts) ? $exts : '');
		}
	}

	private function isVersioned(){
		switch($this->ContentType){
			case we_base_ContentTypes::AUDIO:
				return VERSIONING_AUDIO;
			case we_base_ContentTypes::VIDEO:
				return VERSIONING_VIDEO;
			case we_base_ContentTypes::FLASH://FIXME: remove flash/quicktime, replace by video
				return VERSIONING_FLASH;
			case we_base_ContentTypes::IMAGE:
				return VERSIONING_IMAGE;
			case we_base_ContentTypes::TEMPLATE:
				return VERSIONING_TEXT_WETMPL;
			case we_base_ContentTypes::QUICKTIME:
				return VERSIONING_QUICKTIME;
			case we_base_ContentTypes::JS:
				return VERSIONING_TEXT_JS;
			case we_base_ContentTypes::CSS:
				return VERSIONING_TEXT_CSS;
			case we_base_ContentTypes::TEXT:
				return VERSIONING_TEXT_PLAIN;
			case we_base_ContentTypes::XML:
				return VERSIONING_TEXT_XML;
			default:
			case we_base_ContentTypes::APPLICATION:
				return VERSIONING_SONSTIGE;
		}
	}

	public function we_save($resave = false, $skipHook = false){
		$this->errMsg = '';
		$this->i_setText();

		if(!$skipHook){
			$hook = new weHook('preSave', '', array($this, 'resave' => $resave));
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}

		if(!parent::we_save($resave)){
			return false;
		}
		$ret = $this->i_writeDocument();
		if(!$ret || ($this->errMsg != '')){
			return false;
		}
		$this->OldPath = $this->Path;

		if(!$resave){ // NO rebuild!!!
			$this->resaveWeDocumentCustomerFilter();
		}

		if($this->isVersioned()){
			$version = new we_versions_version();
			$version->save($this);
		}

		/* hook */
		if(!$skipHook){
			$hook = new weHook('save', '', array($this, 'resave' => $resave));
//check if doc should be saved
			if($hook->executeHook() === false){
				$this->errMsg = $hook->getErrorString();
				return false;
			}
		}
		return $ret;
	}

	function resaveWeDocumentCustomerFilter(){
		if(isset($this->documentCustomerFilter) && $this->documentCustomerFilter){
			we_customer_documentFilter::saveForModel($this);
		}
	}

	public function we_load($from = we_class::LOAD_MAID_DB){
		parent::we_load($from);
// Navigation items
		$this->i_setExtensions();
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_textContentDocument::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(){
		$this->documentCustomerFilter = we_customer_documentFilter::getFilterOfDocument($this);
	}

// reverse function to we_init_sessDat
	/* function saveInSession(&$save){
	  parent::saveInSession($save);
	  //$save[2] = $this->NavigationItems;
	  } */

// reverse function to saveInSession !!!
	public function we_initSessDat($sessDat){
		parent::we_initSessDat($sessDat);
		if(we_base_moduleInfo::isActive(we_base_moduleInfo::SCHEDULER)){
			if(
				($day = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_From_day')) && ($month = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_From_month')) && ($year = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_From_year')) && ($hour = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_From_hour')) !== false && ($min = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_From_minute')) !== false){
				$this->From = mktime($hour, $min, 0, $month, $day, $year);
			}
			if(
				($day = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_To_day')) && ($month = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_To_month')) && ($year = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_To_year')) && ($hour = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_To_hour')) !== false && ($min = we_base_request::_(we_base_request::INT, 'we_' . $this->Name . '_To_minute')) !== false){
				$this->To = mktime($hour, $min, 0, $month, $day, $year);
			}
		}
		/* if(isset($sessDat[2])){
		  $this->NavigationItems = $sessDat[2];
		  } else {
		  $this->i_loadNavigationItems();
		  } */


		if(we_base_request::_(we_base_request::INT, 'wecf_mode') !== false){
			$this->documentCustomerFilter = we_customer_documentFilter::getCustomerFilterFromRequest($this->ID, $this->ContentType, $this->Table);
		} else if(isset($sessDat[3])){ // init webUser from session
			$this->documentCustomerFilter = unserialize($sessDat[3]);
		}


		$this->i_setExtensions();

		if(!($this->Language) && $this->Table != TEMPLATES_TABLE){
			$this->initLanguageFromParent();
		}
	}

	function we_rewrite(){
		$this->RebuildDate = time();
		return $this->i_writeDocument();
	}

	/*
	 * private
	 */

	protected function i_setText(){
		$this->Text = $this->Filename . $this->Extension;
	}

	protected function i_writeSiteDir($doc){
		if($this->isMoved()){
			we_base_file::deleteLocalFile($this->getSitePath(true));
		}
		return we_base_file::checkAndMakeFolder(dirname($this->getSitePath()), true) && we_base_file::save($this->getSitePath(), $doc);
	}

	protected function i_writeMainDir($doc){
		if($this->isMoved()){
			we_base_file::deleteLocalFile($this->getRealPath(true));
		}
		return we_base_file::save($this->getRealPath(), $doc);
	}

	protected function i_writeDocument(){
		$update = $this->isMoved();
		$doc = $this->i_getDocumentToSave();
		if(!$this->i_writeSiteDir($doc) || !$this->i_writeMainDir($doc)){
			return false;
		}

		if($update){
			$this->rewriteNavigation();
		}
		$this->update_filehash();
		return true;
	}

	protected function i_getDocumentToSave(){
		$this->DocStream = $this->DocStream ? : $this->i_getDocument();
		return $this->DocStream;
	}

	function i_getDocument($includepath = ''){
		return $this->getElement('data');
	}

	function i_setDocument($value){
		$this->setElement('data', $value);
	}

	function i_filenameDouble(){
		return f('SELECT 1 FROM ' . escape_sql_query($this->Table) . ' WHERE ParentID=' . intval($this->ParentID) . " AND Filename='" . escape_sql_query($this->Filename) . "' AND Extension='" . escape_sql_query($this->Extension) . "' AND ID != " . intval($this->ID), "", $this->DB_WE);
	}

//FIXME: parameter $attrib should be: array $attribs=array()
//FIXME: check if we can rid of this function, since it causes problems every change of tags since it also uses the given attribs array!
	public function getFieldByVal($val, $type, $attribs = '', $pathOnly = false, $parentID = 0, $path = '', we_database_base $db = null, $classID = '', $fn = 'this'){
		$attribs = is_array($attribs) ? $attribs : array();
		if(isset($attribs['_name_orig'])){
			unset($attribs['_name_orig']);
		}
		$db = ($db ? : new DB_WE());
		if((!$attribs) || (!is_array($attribs))){
			$attribs = array();
		}
		switch($type){
			case 'img':
				if(!$val && isset($attribs['id'])){
					$val = $attribs['id'];
				}

				$img = new we_imageDocument(false);

				if(isset($attribs['name'])){
					$img->Name = $attribs['name'];
				}

				$img->initByID($val, FILE_TABLE);

				$altField = $img->Name . we_imageDocument::ALT_FIELD;
				$titleField = $img->Name . we_imageDocument::TITLE_FIELD;

				if(isset($GLOBALS['lv'])){
					switch(get_class($GLOBALS['lv'])){
						case 'we_shop_listviewShopVariants':
							$altField = (WE_SHOP_VARIANTS_PREFIX . $GLOBALS['lv']->Position . '_' . $altField);
							$titleField = (WE_SHOP_VARIANTS_PREFIX . $GLOBALS['lv']->Position . '_' . $titleField);
							break;
						case 'we_listview_document':
							$alt = $GLOBALS['lv']->f($altField) ? : '';
							$title = $GLOBALS['lv']->f($titleField) ? : '';
						case 'we_object_listview':
						case 'we_object_listviewMultiobject':
							$attribs['alt'] = isset($alt) && $alt ? $alt : ($img->getElement('alt') ? : (isset($attribs['alt']) ? $attribs['alt'] : ''));
							$attribs['title'] = isset($title) && $title ? $title : ($img->getElement('title') ? : (isset($attribs['title']) ? $attribs['title'] : ''));
							break;
					}
				}
				if(isset($attribs['alt'])){
					$attribs['alt'] = oldHtmlspecialchars($attribs['alt']);
				}
				if(isset($attribs['title'])){
					$attribs['title'] = oldHtmlspecialchars($attribs['title']);
				}
				if(!(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0) === 'reload_editpage' && ($img->Name === we_base_request::_(we_base_request::STRING, 'we_cmd', false, 1)) && we_base_request::_(we_base_request::STRING, 'we_cmd', '', 2) === 'change_image') && isset($GLOBALS['we_doc']->elements[$altField])){
					if(!isset($GLOBALS['lv'])){
						$attribs['alt'] = oldHtmlspecialchars($GLOBALS['we_doc']->getElement($altField));
						$attribs['title'] = oldHtmlspecialchars($GLOBALS['we_doc']->getElement($titleField));
					}
				}

//	when width or height are given, then let the browser adjust the image
				if(isset($attribs['width'])){
					unset($img->elements['width']);
				}
				if(isset($attribs['height'])){
					unset($img->elements['height']);
				}
				if($attribs){
					$img->initByAttribs(removeAttribs($attribs, array('hyperlink', 'target')));
				}
				if(isset($GLOBALS['lv'])){
					if(isset($GLOBALS['lv']->count)){
						$img->setElement('name', $img->getElement('name') . '_' . $GLOBALS['lv']->count, 'attrib');
						$img->Name = $img->Name . '_' . $GLOBALS['lv']->count;
					} else {
						$img->setElement('name', $img->getElement('name'), 'attrib');
					}
				}
				
				switch($pathOnly ? 'path' : (isset($attribs['only']) ? $attribs['only'] : '')){
					case 'src': //TODO: make separate case for multi domain project to devide between path and src
					case 'path':
						return (isset($attribs['thumbnail']) ? $img->getHtml(false, true, $pathOnly) : $img->Path);
					case 'id':
						return $img->ID;
				}
				
				return $img->getHtml(false, true);
			case 'binary':
				$bin = new we_otherDocument();
				if(isset($attribs['name'])){
					$bin->Name = $attribs['name'];
				}
				if(!$val && isset($attribs['id'])){
					$val = $attribs['id'];
				}
				$bin->initByID($val, FILE_TABLE);
				return array($bin->Text, $bin->Path, $bin->ParentPath, $bin->Filename, $bin->Extension, $bin->getElement('filesize'));
			case 'video':
				$video = new we_document_video();
				if(isset($attribs['name'])){
					$video->Name = $attribs['name'];
				}
				if(!$val && isset($attribs['id'])){
					$val = $attribs['id'];
				}
				$video->initByID($val, FILE_TABLE);
				/* if(!empty($attribs)){
				  $video->initByAttribs($attribs);
				  } */
				return $pathOnly ? $video->Path : $video->getHtml(false, $GLOBALS['we_editmode']);
			case 'flashmovie':
				$fl = new we_flashDocument();
				if(isset($attribs['name'])){
					$fl->Name = $attribs['name'];
				}
				if(!$val && isset($attribs['id'])){
					$val = $attribs['id'];
				}
				$fl->initByID($val, FILE_TABLE);
				if(!empty($attribs)){
					$fl->initByAttribs($attribs);
				}
				return $pathOnly ? $fl->Path : $fl->getHtml();
			case 'quicktime':
				$fl = new we_quicktimeDocument();
				if(isset($attribs['name'])){
					$fl->Name = $attribs['name'];
				}
				if(!$val && isset($attribs['id'])){
					$val = $attribs['id'];
				}
				$fl->initByID($val, FILE_TABLE);
				if(!empty($attribs)){
					$fl->initByAttribs($attribs);
				}
				return $pathOnly ? $fl->Path : $fl->getHtml();
			case 'link':
				$link = $val ? unserialize($val) : array();

				$only = weTag_getAttribute('only', $attribs, '', we_base_request::STRING);

				$hidedirindex = weTag_getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
				$objectseourls = weTag_getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);

				if($pathOnly || $only === 'href'){
					$return = self::getLinkHref($link, $parentID, $path, $db, $hidedirindex, $objectseourls);

					if((isset($GLOBALS['we_link_not_published'])) && ($GLOBALS['we_link_not_published'])){
						unset($GLOBALS['we_link_not_published']);
						return '';
					}
					return $return;
				}

				if(is_array($link)){
					$img = new we_imageDocument();
//	set name of image for rollover ...

					if(isset($attribs['name'])){ //	here we must change the name for a rollover-image
						$_useName = $attribs['name'] . '_img';
						$img->setElement('name', $_useName, 'dat');
					} else {
						$_useName = '';
					}

					$xml = weTag_getAttribute('xml', $attribs, (XHTML_DEFAULT), we_base_request::BOOL);
					$oldHtmlspecialchars = weTag_getAttribute('htmlspecialchars', $attribs, true, we_base_request::BOOL);
					if($only){
						return ($only === 'content' ?
								self::getLinkContent($link, $parentID, $path, $db, $img, $xml, $_useName, $oldHtmlspecialchars, $hidedirindex, $objectseourls) :
								isset($link[$only]) ? $link[$only] : ''); // #3636
					}

					if(($content = self::getLinkContent($link, $parentID, $path, $db, $img, $xml, $_useName, $oldHtmlspecialchars, $hidedirindex, $objectseourls))){
						if(($startTag = self::getLinkStartTag($link, $attribs, $parentID, $path, $db, $img, $_useName, $hidedirindex, $objectseourls))){
							return $startTag . $content . '</a>';
						}
						return $content;
					}
				}
				return '';
			case 'date':
				$val = $val ? : time();
				$format = isset($attribs['format']) && $attribs['format'] ? $attribs['format'] : g_l('date', '[format][default]');
				Zend_Registry::set('Zend_Locale', new Zend_Locale((isset($GLOBALS['WE_MAIN_DOC']) && $GLOBALS['WE_MAIN_DOC']->Language ? $GLOBALS['WE_MAIN_DOC']->Language : $GLOBALS['weDefaultFrontendLanguage'])));
				$zdate = is_numeric($val) ? new Zend_Date($val, Zend_Date::TIMESTAMP) : new Zend_Date($val);

//workaround buggy zend dateformat with \h which duplicates the char
				$ret = '';
				for($i = 0; $i < strlen($format); $i++){
					$ret.=($format[$i] === '\\' ?
							$format[++$i] :
							$zdate->toString($format[$i], 'php')
						);
				}
				return $ret;

			case 'select':
				if(defined('OBJECT_TABLE')){
					if(strlen($val) == 0){
						return '';
					}
					if($classID){
						$defVals = f('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classID), '', $db);
						if($defVals){
							$arr = unserialize($defVals);
							return isset($arr['meta_' . $attribs['name']]['meta'][$val]) ? $arr['meta_' . $attribs['name']]['meta'][$val] : '';
						}
					}
				}
				$f = __FUNCTION__;
				return $this->{$f}($val, 'text', $attribs, $pathOnly, $parentID, $path, $db, $classID, $fn);
			case 'href':
				return $this->getHref($attribs, $db, $fn);
			default:
				self::parseInternalLinks($val, $parentID);
				$retval = preg_replace('/<\?xml[^>]+>/i', '', $val);

				if(!weTag_getAttribute('html', $attribs, true, we_base_request::BOOL) && !weTag_getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL)){
					$retval = strip_tags($retval, '<br/>,<p>');
				}

				$_htmlspecialchars = weTag_getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL);
				$_wysiwyg = weTag_getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL);

				if($_htmlspecialchars && (!$_wysiwyg)){
					$retval = preg_replace('/#we##br([^#]*)#we##/', '<br${1}>', oldHtmlspecialchars(preg_replace('/<br([^>]*)>/i', '#we##br${1}#we##', $retval), ENT_QUOTES));
				}
				if(!weTag_getAttribute('php', $attribs, (defined('WE_PHP_DEFAULT') && WE_PHP_DEFAULT), we_base_request::BOOL)){
					$retval = we_base_util::rmPhp($retval);
				}
				$xml = weTag_getAttribute('xml', $attribs, (XHTML_DEFAULT), we_base_request::BOOL);
				$retval = preg_replace('-<(br|hr)([^/>]*)/? *>-i', ($xml ? '<${1}${2}/>' : '<${1}${2}>'), $retval);

				if(preg_match('/^[\d.,]+$/', trim($retval))){
					$precision = isset($attribs['precision']) ? abs($attribs['precision']) : 2;
					if(($num = weTag_getAttribute('num_format', $attribs, '', we_base_request::STRING))){
						$retval = we_util_Strings::formatNumber(we_base_util::std_numberformat($retval), $num, $precision);
					}
				}
				if(weTag_getAttribute('win2iso', $attribs, false, we_base_request::BOOL)){

					$charset = ( isset($GLOBALS['WE_MAIN_DOC']) && isset($GLOBALS['WE_MAIN_DOC']->elements['Charset']['dat'])) ? $GLOBALS['WE_MAIN_DOC']->elements['Charset']['dat'] : '';
					if(trim(strtolower(substr($charset, 0, 3))) === 'iso' || $charset === ''){
						$retval = strtr($retval, array(
							chr(128) => '&#8364;',
							chr(130) => '&#8218;',
							chr(131) => '&#402;',
							chr(132) => '&#8222;',
							chr(133) => '&#8230;',
							chr(134) => '&#8224;',
							chr(135) => '&#8225;',
							chr(136) => '&#710;',
							chr(137) => '&#8240;',
							chr(138) => '&#352;',
							chr(139) => '&#8249;',
							chr(140) => '&#338;',
							chr(142) => '&#381;',
							chr(145) => '&#8216;',
							chr(146) => '&#8217;',
							chr(147) => '&#8220;',
							chr(148) => '&#8221;',
							chr(149) => '&#8226;',
							chr(150) => '&#8211;',
							chr(151) => '&#8212;',
							chr(152) => '&#732;',
							chr(153) => '&#8482;',
							chr(154) => '&#353;',
							chr(155) => '&#8250;',
							chr(156) => '&#339;',
							chr(158) => '&#382;',
							chr(159) => '&#376;'));
					}
				}
				return str_replace(array("##|n##", "##|r##"), array("\n", "\r"), $retval);
		}
	}

	function getField($attribs, $type = 'txt', $pathOnly = false){
		if(is_array($attribs) && isset($attribs['_name_orig'])){
			unset($attribs['_name_orig']);
		}

		$val = '';
		switch($type){
			case 'img':
			case 'flashmovie':
			case 'quicktime':
			case 'video':
				if(isset($attribs['showcontrol']) && !$attribs['showcontrol'] && isset($attribs['id']) && $attribs['id']){//bug 6433: siehe korrespondierende Änderung in we_tag_img
					unset($attribs['showcontrol']);
					$val = $attribs['id'];
					break;
				}
				$val = $this->getElement($attribs['name']);
				break;
			case 'href':
				if(!isset($attribs['name'])){
					return;
				}
				$val = $this->getElement($attribs['name']);
				if($this instanceof we_objectFile || (is_string($val) && $val && $val{0} == 'a')){
					$hrefArr = $val ? unserialize($val) : array();
					return (is_array($hrefArr) ? self::getHrefByArray($hrefArr) : '');
				}
				break;
			default:
//check bdid first
				$val = $this->getElement($attribs['name']);
		}

		return $this->getFieldByVal($val, $type, $attribs, $pathOnly, isset($GLOBALS['WE_MAIN_DOC']) ? $GLOBALS['WE_MAIN_DOC']->ParentID : $this->ParentID, isset($GLOBALS['WE_MAIN_DOC']) ? $GLOBALS['WE_MAIN_DOC']->Path : $this->Path, $this->DB_WE, (isset($attribs['classid']) && isset($attribs['type']) && $attribs['type'] === 'select') ? $attribs['classid'] : (isset($this->TableID) ? $this->TableID : '')); //not instance due to we_showObject
	}

	private function getValFromSrc($fn, $name, $key = 'dat'){
		switch($fn){
			default:
			case 'this':
				return $this->getElement($name, $key);
			case 'listview':
				return $GLOBALS['lv']->f($name);
		}
	}

	function getHref($attribs, we_database_base $db = null, $fn = 'this'){
		$db = $db ? : new_DB_WE();
		$n = $attribs['name'];
		$nint = $n . we_base_link::MAGIC_INT_LINK;
		if($this->getValFromSrc($fn, $nint)){
			$intID = $this->getValFromSrc($fn, $n . we_base_link::MAGIC_INT_LINK_ID, 'bdid'); //try bdid first
			$intID = $intID ? : $this->getValFromSrc($fn, $n . we_base_link::MAGIC_INT_LINK_ID);
			return f('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($intID), '', $db);
		}
		return $this->getValFromSrc($fn, $n);
	}

	static function getHrefByArray($hrefArr){
		if(isset($hrefArr['int']) && $hrefArr['int']){
			$intID = isset($hrefArr['intID']) ? $hrefArr['intID'] : 0;
			return $intID ? id_to_path($intID) : '';
		}
		return isset($hrefArr['extPath']) ? $hrefArr['extPath'] : '';
	}

	function getLinkHref($link, $parentID, $path, we_database_base $db = null, $hidedirindex = false, $objectseourls = false){
		$db = ($db ? : new DB_WE());

// Bug Fix 8170&& 8166
		if(isset($link['href']) && strpos($link['href'], we_base_link::TYPE_MAIL_PREFIX) === 0){
			$link['type'] = we_base_link::TYPE_MAIL;

//added for #7269
			if(isset($link['subject']) && $link['subject'] != ''){
				$link['href'] = $link['href'] . "?subject=" . $link['subject'];
			}
			if(isset($link['cc']) && $link['cc'] != ''){
				$link['href'] = $link['href'] . "&cc=" . $link['cc'];
			}
			if(isset($link['bcc']) && $link['bcc'] != ''){
				$link['href'] = $link['href'] . "&bcc=" . $link['bcc'];
			}
		}
		if(!isset($link['type'])){
			return '';
		}
		switch($link['type']){
			case we_base_link::TYPE_INT:
				$id = $link['id'];
				if(!$id){
					return '';
				}
				$path = f('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id), '', $db);
				$path_parts = pathinfo($path);
				if($hidedirindex && show_SeoLinks() && NAVIGATION_DIRECTORYINDEX_NAMES && in_array($path_parts['basename'], array_map('trim', explode(',', NAVIGATION_DIRECTORYINDEX_NAMES)))){
					$path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
				}
				if(isset($GLOBALS['we_doc']) && $GLOBALS['we_doc']->InWebEdition || f('SELECT Published FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id), '', $db)){
					return $path;
				}
				$GLOBALS['we_link_not_published'] = 1;
				return '';

			case we_base_link::TYPE_OBJ:
				return we_objectFile::getObjectHref($link['obj_id'], $parentID, $path, $db, $hidedirindex, $objectseourls);
			default:
				return ($link['href'] == we_base_link::EMPTY_EXT ? '' : $link['href']);
		}
	}

	function getLinkContent($link, $parentID = 0, $path = '', we_database_base $db = null, $img = '', $xml = '', $_useName = '', $htmlspecialchars = false, $hidedirindex = false, $objectseourls = false){
		$l_href = self::getLinkHref($link, $parentID, $path, $db, $hidedirindex, $objectseourls);

		if(isset($GLOBALS['we_link_not_published']) && $GLOBALS['we_link_not_published']){
			unset($GLOBALS['we_link_not_published']);
			return '';
		}

		switch(isset($link['ctype']) ? $link['ctype'] : ''){
			case we_base_link::CONTENT_INT:
				$img = ($img ? : new we_imageDocument());
				$img->initByID($link['img_id']);

				$img_attribs = array('width' => $link['width'], 'height' => $link['height'], 'border' => $link['border'], 'hspace' => $link['hspace'], 'vspace' => $link['vspace'], 'align' => $link['align'], 'alt' => $link['alt'], 'title' => (isset($link['img_title']) ? $link['img_title'] : ''));

				if($_useName){ //	rollover with links ...
					$img_attribs['name'] = $_useName;
					$img->elements['name']['dat'] = $_useName;
				}

				if($xml){
					$img_attribs['xml'] = 'true';
				}

				$img->initByAttribs($img_attribs);

				return $img->getHtml(false, false);
			case we_base_link::CONTENT_EXT:
//  set default atts
				$img_attribs = array('src' => $link['img_src'],
					'alt' => '',
					'xml' => $xml
				);
				if(isset($link['img_title'])){
					$img_attribs['title'] = $link['img_title'];
				}
//  deal with all remaining attribs
				$img_attList = array('width', 'height', 'border', 'hspace', 'vspace', 'align', 'alt', 'name');
				foreach($img_attList as $k){
					if(isset($link[$k]) && $link[$k] != ''){
						$img_attribs[$k] = $link[$k];
					}
				}
				return getHtmlTag('img', $img_attribs);
			case we_base_link::CONTENT_TEXT:
// Workarround => We have to find another solution
				return (($xml || $htmlspecialchars) ?
						oldHtmlspecialchars(html_entity_decode($link['text'])) : $link['text']);
		}
	}

	function getLinkStartTag($link, $attribs, $parentID = 0, $path = '', we_database_base $db = null, $img = '', $_useName = '', $hidedirindex = false, $objectseourls = false){
		if(($l_href = self::getLinkHref($link, $parentID, $path, $db, $hidedirindex, $objectseourls))){
//    define some arrays to order the attribs to image, link or js-window ...
			$_popUpAtts = array('jswin', 'jscenter', 'jswidth', 'jsheight', 'jsposx', 'jsposy', 'jsstatus', 'jsscrollbars', 'jsmenubar', 'jstoolbar', 'jsresizable', 'jslocation');

//    attribs only for image - these are already handled
			$_imgAtts = array('img_id', 'width', 'height', 'border', 'hspace', 'vspace', 'align', 'alt', 'img_title');

//    these are handled separately
			$_dontUse = array('img_id', 'obj_id', 'ctype', 'anchor', 'params', 'attribs', 'img_src', 'text', 'type', 'only');

//    these are already handled dont get them in output
			$_we_linkAtts = array('id');

			$_linkAttribs = array();

// define image-if necessary - handle with image-attribs
			$img = ($img ? : new we_imageDocument());

//   image attribs
			foreach($_imgAtts as $att){ //  take all attribs belonging to image inside content
				$img_attribs[$att] = isset($link[$att]) ? $link[$att] : '';
			}

			$img->initByID($img_attribs['img_id']);
			$img->initByAttribs($img_attribs);


			if($link['ctype'] == we_base_link::TYPE_INT){
//	set name of image dynamically
				if($_useName){ //	we must set the name of the image -> rollover
					$img->setElement('name', $_useName, 'dat');
				}
				$rollOverScript = $img->getRollOverScript();
				$rollOverAttribsArr = $img->getRollOverAttribsArr();
			} else {
				$rollOverScript = '';
				$rollOverAttribsArr = array();
			}

// Link-Attribs
//   1st attribs-string from link dialog ! These are already used in content ...
			if(isset($link['attribs'])){
				$_linkAttribs = array_merge(we_tag_tagParser::makeArrayFromAttribs($link['attribs']), $_linkAttribs);
			}

//   2nd take all atts given in link-array - from function we_tag_link()
			foreach($link as $k => $v){ //   define all attribs - later we can remove/overwrite them
				if($v != '' && !in_array($k, $_we_linkAtts) && !in_array($k, $_imgAtts) && !in_array($k, $_popUpAtts) && !in_array($k, $_dontUse)){
					$_linkAttribs[$k] = $v;
				}
			}

//   3rd we take attribs given from we:link,
			foreach($attribs as $k => $v){ //   define all attribs - later we can remove/overwrite them
				if($v != '' && !in_array($k, $_imgAtts) && !in_array($k, $_popUpAtts) && !in_array($k, $_dontUse)){
					$_linkAttribs[$k] = $v;
				}
			}

//   4th use Rollover attributes
			foreach($rollOverAttribsArr as $n => $v){
				$_linkAttribs[$n] = $v;
			}
//   override the href at last important !!

			$linkAdds = (isset($link['params']) ? $link['params'] : '' ) . (isset($link['anchor']) ? $link['anchor'] : '' );
			if(strpos($linkAdds, '?') === false && strpos($linkAdds, '&') !== false && strpos($linkAdds, '&') == 0){//Bug #5478
				$linkAdds = substr_replace($linkAdds, '?', 0, 1);
			}
			$_linkAttribs['href'] = $l_href . str_replace('&', '&amp;', $linkAdds);

// The pop-up-window                              */
			$_popUpCtrl = array();
			foreach($_popUpAtts as $n){
				if(isset($link[$n])){
					$_popUpCtrl[$n] = $link[$n];
				}
			}


			if(isset($_popUpCtrl['jswin']) && $_popUpCtrl['jswin']){ //  add attribs for popUp-window
				$js = 'var we_winOpts = \'\';';
				if(isset($_popUpCtrl["jscenter"]) && $_popUpCtrl["jscenter"] && isset($_popUpCtrl["jswidth"]) && $_popUpCtrl["jswidth"] && isset($_popUpCtrl["jsheight"]) && $_popUpCtrl["jsheight"]){
					$js .= 'if (window.screen) {var w = ' . $_popUpCtrl["jswidth"] . ';var h = ' . $_popUpCtrl["jsheight"] . ';var screen_height = screen.availHeight - 70;var screen_width = screen.availWidth-10;var w = Math.min(screen_width,w);var h = Math.min(screen_height,h);var x = (screen_width - w) / 2;var y = (screen_height - h) / 2;we_winOpts = \'left=\'+x+\',top=\'+y;}else{we_winOpts=\'\';};';
				} else if((isset($_popUpCtrl["jsposx"]) && $_popUpCtrl["jsposx"] != "") || (isset($_popUpCtrl["jsposy"]) && $_popUpCtrl["jsposy"] != "")){
					if($_popUpCtrl["jsposx"] != ''){
						$js .= 'we_winOpts += (we_winOpts ? \',\' : \'\')+\'left=' . $_popUpCtrl["jsposx"] . '\';';
					}
					if($_popUpCtrl["jsposy"] != ''){
						$js .= 'we_winOpts += (we_winOpts ? \',\' : \'\')+\'top=' . $_popUpCtrl["jsposy"] . '\';';
					}
				}
				$js.=
					(isset($_popUpCtrl["jswidth"]) && $_popUpCtrl["jswidth"] ?
						'we_winOpts += (we_winOpts ? \',\' : \'\')+\'width=' . $_popUpCtrl["jswidth"] . '\';' : '') .
					(isset($_popUpCtrl["jsheight"]) && $_popUpCtrl["jsheight"] ?
						'we_winOpts += (we_winOpts ? \',\' : \'\')+\'height=' . $_popUpCtrl["jsheight"] . '\';' : '') . 'we_winOpts += (we_winOpts ? \',\' : \'\')+\'status=' . (isset($_popUpCtrl["jsstatus"]) && $_popUpCtrl["jsstatus"] ? 'yes' : 'no') . '\';' .
					'we_winOpts += (we_winOpts ? \',\' : \'\')+\'scrollbars=' . (isset($_popUpCtrl["jsscrollbars"]) && $_popUpCtrl["jsscrollbars"] ? 'yes' : 'no') . '\';' .
					'we_winOpts += (we_winOpts ? \',\' : \'\')+\'menubar=' . (isset($_popUpCtrl["jsmenubar"]) && $_popUpCtrl["jsmenubar"] ? 'yes' : 'no') . '\';' .
					'we_winOpts += (we_winOpts ? \',\' : \'\')+\'resizable=' . (isset($_popUpCtrl["jsresizable"]) && $_popUpCtrl["jsresizable"] ? 'yes' : 'no') . '\';' .
					'we_winOpts += (we_winOpts ? \',\' : \'\')+\'location=' . (isset($_popUpCtrl["jslocation"]) && $_popUpCtrl["jslocation"] ? 'yes' : 'no') . '\';' .
					'we_winOpts += (we_winOpts ? \',\' : \'\')+\'toolbar=' . (isset($_popUpCtrl["jstoolbar"]) && $_popUpCtrl["jstoolbar"] ? 'yes' : 'no') . '\';';
				$foo = $js . "var we_win = window.open('','we_" . (isset($attribs["name"]) ? $attribs["name"] : "") . "',we_winOpts);";

				$_linkAttribs['target'] = 'we_' . (isset($attribs["name"]) ? $attribs["name"] : "");
				$_linkAttribs['onclick'] = $foo;
			}
			$_linkAttribs = removeAttribs($_linkAttribs, array('hidedirindex', 'objectseourls'));
			return $rollOverScript . getHtmlTag('a', $_linkAttribs, '', false, true);
		} else {
			if((isset($GLOBALS['we_link_not_published'])) && ($GLOBALS['we_link_not_published'])){
				unset($GLOBALS['we_link_not_published']);
			}
		}
	}

	/*
	 * functions for scheduler pro
	 */

	function add_schedule(){
		$this->schedArr[] = we_schedpro::getEmptyEntry();
	}

	function del_schedule($nr){
		unset($this->schedArr[$nr]);
	}

	protected function i_setElementsFromHTTP(){
		parent::i_setElementsFromHTTP();
		if($_REQUEST){
			$dates = $regs = array();
			foreach($_REQUEST as $n => $v){
				if(preg_match('/^we_schedule_([^\[]+)$/', $n, $regs)){//make this an array
					$rest = $regs[1];
					$sw = explode('_', $rest);
					$nr = end($sw);
					if(!isset($this->schedArr[$nr])){
						$this->schedArr[$nr] = array();
					}
					switch($sw[0]){
						case 'task':
							$this->schedArr[$nr]['task'] = $v;
							break;
						case 'type':
							$this->schedArr[$nr]['type'] = $v;
							break;
						case 'active':
							$this->schedArr[$nr]['active'] = $v;
							break;
						case 'doctype':
							$this->schedArr[$nr]['DoctypeID'] = $v;
							break;
						case 'doctypeAll':
							$this->schedArr[$nr]['doctypeAll'] = $v;
							break;
						case 'parentid':
							$this->schedArr[$nr]['ParentID'] = $v;
							break;
						case 'time':
							if((!isset($dates[$nr]) || !is_array($dates[$nr]))){
								$dates[$nr] = array();
							}
							$dates[$nr][$sw[1]] = $v;
							break;
						default:
							if(substr($sw[0], 0, 5) === 'month'){
								$d = intval(substr($sw[0], 5));
								$this->schedArr[$nr]['months'][$d - 1] = $v;
							} else if(substr($sw[0], 0, 3) === 'day'){
								$d = intval(substr($sw[0], 3));
								$this->schedArr[$nr]['days'][$d - 1] = $v;
							} else if(substr($sw[0], 0, 4) === 'wday'){
								$d = intval(substr($sw[0], 4));
								$this->schedArr[$nr]['weekdays'][$d - 1] = $v;
							}
					}
				}
			}
			foreach($dates as $nr => $v){
				$this->schedArr[$nr]['time'] = mktime(
					$dates[$nr]['hour'], $dates[$nr]['minute'], 0, $dates[$nr]['month'], $dates[$nr]['day'], $dates[$nr]['year']);
			}
		}
		$this->Path = $this->getPath();
	}

	function add_schedcat($id, $nr){
		$cats = makeArrayFromCSV($this->schedArr[$nr]['CategoryIDs']);
		if(!in_array($id, $cats)){
			$cats[] = $id;
		}
		$this->schedArr[$nr]['CategoryIDs'] = makeCSVFromArray($cats, true);
	}

	function delete_schedcat($id, $nr){
		$cats = makeArrayFromCSV($this->schedArr[$nr]['CategoryIDs']);
		if(in_array($id, $cats)){
			$pos = array_search($id, $cats);
			if($pos !== false){
				unset($cats[$pos]);
			}
		}
		$this->schedArr[$nr]['CategoryIDs'] = makeCSVFromArray($cats, true);
	}

// returns the next date when the document gets published
	function getNextPublishDate(){
		$times = array();
		foreach($this->schedArr as $s){
			if($s['task'] == we_schedpro::SCHEDULE_FROM && $s['active']){
				$times[] = we_schedpro::getNextTimestamp($s, time());
			}
		}
		if(!$times){
			return 0;
		}
		sort($times);
		return $times[0];
	}

	function loadSchedule(){
		if(we_base_moduleInfo::isActive(we_base_moduleInfo::SCHEDULER)){
			$this->DB_WE->query('SELECT * FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND ClassName="' . $this->DB_WE->escape($this->ClassName) . '"');
			if($this->DB_WE->num_rows()){
				$this->schedArr = array();
			}
			while($this->DB_WE->next_record()){
				$s = we_unserialize($this->DB_WE->f('Schedpro'));
				if(is_array($s)){
					$s['active'] = $this->DB_WE->f('Active');
					$this->schedArr[] = $s;
				}
			}
		}
	}

	/**
	 * returns	a select menu within a html table. to ATTENTION this function is also used in classes object and objectFile !!!!
	 * 			when $withHeadline is true, a table with headline is returned, default is false
	 * @return	select menue to determine charset
	 * @param	boolean
	 */
	function formCharset($withHeadline = false){
		$value = $this->getElement('Charset');

		$_charsetHandler = new we_base_charsetHandler();

		$_charsets = $_charsetHandler->getCharsetsForTagWizzard();
		$_charsets[''] = '';
		asort($_charsets);
		reset($_charsets);

		$name = 'Charset';

		$inputName = 'we_' . $this->Name . "_txt[$name]";


		return '<table border="0" cellpadding="0" cellspacing="0">' .
			($withHeadline ? '<tr><td class="defaultfont">' . g_l('weClass', '[Charset]') . '</td></tr>' : '') .
			'<tr><td>' . $this->htmlTextInput($inputName, 24, $value) . '</td><td></td><td>' . $this->htmlSelect('we_tmp_' . $this->Name . '_select[' . $name . ']', $_charsets, 1, $value, false, array("onblur" => "_EditorFrame.setEditorIsHot(true);document.forms[0].elements['" . $inputName . "'].value=this.options[this.selectedIndex].value;top.we_cmd('reload_editpage');", "onchange" => "_EditorFrame.setEditorIsHot(true);document.forms[0].elements['" . $inputName . "'].value=this.options[this.selectedIndex].value;top.we_cmd('reload_editpage');"), "value", 330) . '</td></tr>' .
			'</table>';
	}

	/**
	 * returns if document can have variants the function returns true otherwise
	 * false
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants($checkFields = false){
// overwrite
		return false;
	}

	/**
	 * @return	array with the filed names and attributes
	 * @param	none
	 */
	function getVariantFields(){
// overwrite
		return array();
	}

	/**
	 * @desc	the function modifies document EditPageNrs set
	 */
	function checkTabs(){
		if(!$this->canHaveVariants(true)){

			$ind = array_search(we_base_constants::WE_EDITPAGE_VARIANTS, $this->EditPageNrs);
			if(!empty($ind)){
				unset($this->EditPageNrs[$ind]);
			}
		}
	}

	/* private function i_deleteNavigation(){
	  $this->DB_WE->query('DELETE FROM ' . NAVIGATION_TABLE . ' WHERE ' . we_navigation_navigation::getNavCondition($this->ID, $this->Table));
	  return true;
	  } */

	/**
	 * get styles for textarea or object
	 * @return type
	 */
	public function getDocumentCss(){
		return $this->CSS;
	}

	/**
	 * get styles for textarea or object
	 * @return type
	 */
	public function addDocumentCss($stylesheet){
		$this->CSS[] = $stylesheet;
	}

	protected function update_filehash(){
		switch($this->Table){
			default:
				return;
			case TEMPLATES_TABLE:
			case FILE_TABLE:
		}
		$this->wasUpdate = $this->ID > 0;
		$usepath = ($this->Table == TEMPLATES_TABLE ?
				(strpos($this->Path, '.tmpl') === false ? TEMPLATES_PATH . $this->Path : TEMPLATES_PATH . substr_replace($this->Path, '.php', -5)) :
				$_SERVER['DOCUMENT_ROOT'] . $this->Path
			);

		$this->Filehash = (file_exists($usepath) && is_file($usepath) ? sha1_file($usepath) : '');
		$this->i_savePersistentSlotsToDB('Filehash,RebuildDate');
	}

	public static function parseInternalLinks(&$text, $pid, $path = ''){
		$DB_WE = new DB_WE();
		$regs = array();
		if(preg_match_all('/(href|src)="(' . we_base_link::TYPE_INT_PREFIX . '|\?id=)(\\d+)(&amp;|&)?("|[^"]+")/i', $text, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				$foo = getHash('SELECT Path,(ContentType="' . we_base_ContentTypes::IMAGE . '") AS isImage  FROM ' . FILE_TABLE . ' WHERE ID=' . intval($reg[3]) . (isset($GLOBALS['we_doc']->InWebEdition) && $GLOBALS['we_doc']->InWebEdition ? '' : ' AND Published>0'), $DB_WE);

				if($foo && $foo['Path']){
					$path_parts = pathinfo($foo['Path']);
					if(show_SeoLinks() && WYSIWYGLINKS_DIRECTORYINDEX_HIDE && NAVIGATION_DIRECTORYINDEX_NAMES && in_array($path_parts['basename'], array_map('trim', explode(',', NAVIGATION_DIRECTORYINDEX_NAMES)))){
						$foo['Path'] = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
					}
					$text = str_replace($reg[1] . '="' . $reg[2] . $reg[3] . $reg[4] . $reg[5], $reg[1] . '="' . $foo['Path'] . ($reg[4] ? '?' : '') . $reg[5], $text);
				} else {
					$text = preg_replace(array(
						'-<(a|img) [^>]*' . $reg[1] . '="' . $reg[2] . $reg[3] . '("|&|&amp;|\?)[^>]*>(.*)</a>-Ui',
						'-<(a|img) [^>]*' . $reg[1] . '="' . $reg[2] . $reg[3] . '(\?|&|&amp;|")[^>]*>-Ui',
						), array(
						'$3',
						''
						), $text);
				}
			}
		}
		if(preg_match_all('/src="' . we_base_link::TYPE_THUMB_PREFIX . '(\d+),(\d+)"/i', $text, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				list(, $imgID, $thumbID) = $reg;
				$thumbObj = new we_thumbnail();
				if($thumbObj->initByImageIDAndThumbID($imgID, $thumbID)){
					$text = str_replace('src="' . we_base_link::TYPE_THUMB_PREFIX . $imgID . ',' . $thumbID . '"', 'src="' . $thumbObj->getOutputPath(false, true) . '"', $text);
				} else {
					$text = preg_replace('|<img[^>]+src="' . we_base_link::TYPE_THUMB_PREFIX . $imgID . ',' . $thumbID . '"[^>]+>|Ui', '', $text);
				}
			}
		}
		if(defined('OBJECT_TABLE')){
			if(preg_match_all('/href="' . we_base_link::TYPE_OBJ_PREFIX . '(\d+)(\??)("|[^"]+")/i', $text, $regs, PREG_SET_ORDER)){
				foreach($regs as $reg){
					$href = we_objectFile::getObjectHref($reg[1], $pid, $path, $DB_WE, WYSIWYGLINKS_DIRECTORYINDEX_HIDE, WYSIWYGLINKS_OBJECTSEOURLS);
					if(isset($GLOBALS['we_link_not_published'])){
						unset($GLOBALS['we_link_not_published']);
					}
					if($href){
						$text = ($reg[2] === '?' ?
								str_replace('href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '?', 'href="' . $href . '&amp;', $text) :
								str_replace('href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . $reg[2] . $reg[3], 'href="' . $href . $reg[2] . $reg[3], $text));
					} else {
						$text = preg_replace(array(
							'-<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '("|&|&amp;|\?)[^>]*>(.*)</a>-Ui',
							'-<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '("|&|&amp;|\?)[^>]*>-Ui',
							), array(
							'$2',
							''
							), $text);
					}
				}
			}
		}

		return preg_replace('/\<a>(.*)\<\/a>/siU', '$1', $text);
	}

	private function getNavigationItems(){
		if($this->Table == FILE_TABLE && $this->ID && $this->InWebEdition){
			$this->DB_WE->query('SELECT Path FROM ' . NAVIGATION_TABLE . ' WHERE ((Selection="' . we_navigation_navigation::SELECTION_STATIC . '" AND SelectionType="' . we_navigation_navigation::STPYE_DOCLINK . '") OR (IsFolder=1 AND FolderSelection="' . we_navigation_navigation::STPYE_DOCLINK . '")) AND LinkID=' . intval($this->ID));
			return $this->DB_WE->getAll(true);
		}
		return array();
	}

	/**
	 * Get data which was saved persistent in this session by the editor
	 * @param string $name name of the variable to save
	 * @return mixed false if not found, data if found
	 */
	public function getEditorPersistent($name){
		return isset($this->editorSaves[$name]) ? $this->editorSaves[$name] : false;
	}

	/**
	 * Save data in editor persistent in session
	 * @param string $name name of the variable to save
	 * @param mixed $value the data to save. Don't save objects! Just data & arrays.
	 */
	public function setEditorPersistent($name, $value){
		$this->editorSaves[$name] = $value;
	}

	protected static function makeBlockName($block, $field){
		$block = str_replace('[0-9]+', '####BLOCKNR####', $block);
		$field = str_replace('[0-9]+', '####BLOCKNR####', $field);
		$out = preg_quote($field . 'blk_' . $block . '__') . '[0-9]+';
		return str_replace('####BLOCKNR####', '[0-9]+', $out);
	}

	protected static function makeLinklistName($block, $field){
		$block = str_replace('[0-9]+', '####BLOCKNR####', $block);
		$field = str_replace('[0-9]+', '####BLOCKNR####', $field);
		$out = preg_quote($field . $block . '_TAGS_') . '[0-9]+';
		return str_replace('####BLOCKNR####', '[0-9]+', $out);
	}

}
